@@ -27,6 +27,9 @@ class Adapter(object):
     particular client that is using the session. An adapter provides a wrapper
     of client local data around the global session object.
 
+    version, min_version, max_version and default_microversion can all be
+    given either as a string or a tuple.
+
     :param session: The session object to wrap.
     :type session: keystoneauth1.session.Session
     :param str service_type: The default service_type for URL discovery.
@@ -35,7 +38,9 @@ class Adapter(object):
     :param str region_name: The default region_name for URL discovery.
     :param str endpoint_override: Always use this endpoint URL for requests
                                   for this client.
-    :param tuple version: The version that this API targets.
+    :param version: The minimum version restricted to a given Major API.
+                    Mutually exclusive with min_version and max_version.
+                    (optional)
     :param auth: An auth plugin to use instead of the session one.
     :type auth: keystoneauth1.plugin.BaseAuthPlugin
     :param str user_agent: The User-Agent string to set.
@@ -64,6 +69,19 @@ class Adapter(object):
                                   ``req-$uuid``) that will be passed on all
                                   requests. Enables cross project request id
                                   tracking.
+    :param min_version: The minimum major version of a given API, intended to
+                        be used as the lower bound of a range with
+                        max_version. Mutually exclusive with version.
+                        If min_version is given with no max_version it is as
+                        if max version is 'latest'. (optional)
+    :param max_version: The maximum major version of a given API, intended to
+                        be used as the upper bound of a range with min_version.
+                        Mutually exclusive with version. (optional)
+    :param default_microversion: The default microversion value to send
+                                 with API requests. While microversions are
+                                 a per-request feature, a user may know they
+                                 want to default to sending a specific value.
+                                 (optional)
     """
 
     client_name = None
@@ -76,7 +94,13 @@ class Adapter(object):
                  connect_retries=None, logger=None, allow={},
                  additional_headers=None, client_name=None,
                  client_version=None, allow_version_hack=None,
-                 global_request_id=None):
+                 global_request_id=None,
+                 min_version=None, max_version=None,
+                 default_microversion=None):
+        if version and (min_version or max_version):
+            raise TypeError(
+                "version is mutually exclusive with min_version and"
+                " max_version")
         # NOTE(jamielennox): when adding new parameters to adapter please also
         # add them to the adapter call in httpclient.HTTPClient.__init__ as
         # well as to load_adapter_from_argparse below if the argument is
@@ -96,6 +120,9 @@ class Adapter(object):
         self.allow = allow
         self.additional_headers = additional_headers or {}
         self.allow_version_hack = allow_version_hack
+        self.min_version = min_version
+        self.max_version = max_version
+        self.default_microversion = default_microversion
 
         self.global_request_id = global_request_id
 
@@ -115,6 +142,10 @@ class Adapter(object):
             kwargs.setdefault('region_name', self.region_name)
         if self.version:
             kwargs.setdefault('version', self.version)
+        if self.min_version:
+            kwargs.setdefault('min_version', self.min_version)
+        if self.max_version:
+            kwargs.setdefault('max_version', self.max_version)
         if self.allow_version_hack is not None:
             kwargs.setdefault('allow_version_hack', self.allow_version_hack)
 
@@ -135,6 +166,8 @@ class Adapter(object):
             kwargs.setdefault('logger', self.logger)
         if self.allow:
             kwargs.setdefault('allow', self.allow)
+        if self.default_microversion is not None:
+            kwargs.setdefault('microversion', self.default_microversion)
 
         if isinstance(self.session, (session.Session, Adapter)):
             # these things are unsupported by keystoneclient's session so be
@@ -192,6 +225,27 @@ class Adapter(object):
         self._set_endpoint_filter_kwargs(kwargs)
         return self.session.get_endpoint(auth or self.auth, **kwargs)
 
+    def get_endpoint_data(self, auth=None):
+        """Get the endpoint data for this Adapter's endpoint.
+
+        :param auth: The auth plugin to use for token. Overrides the plugin on
+                     the session. (optional)
+        :type auth: keystoneauth1.plugin.BaseAuthPlugin
+
+        :raises keystoneauth1.exceptions.auth_plugins.MissingAuthPlugin: if a
+            plugin is not available.
+        :raises TypeError: If arguments are invalid
+
+        :returns: Endpoint data if available or None.
+        :rtype: keystoneauth1.discover.EndpointData
+        """
+        kwargs = {}
+        self._set_endpoint_filter_kwargs(kwargs)
+        if self.endpoint_override:
+            kwargs['endpoint_override'] = self.endpoint_override
+
+        return self.session.get_endpoint_data(auth or self.auth, **kwargs)
+
     def invalidate(self, auth=None):
         """Invalidate an authentication plugin."""
         return self.session.invalidate(auth or self.auth)
