@@ -18,7 +18,6 @@ import threading
 
 from positional import positional
 import six
-from six.moves import urllib
 
 from keystoneauth1 import _utils as utils
 from keystoneauth1 import access
@@ -44,7 +43,7 @@ class BaseIdentityPlugin(plugin.BaseAuthPlugin):
         self.auth_ref = None
         self.reauthenticate = reauthenticate
 
-        self._endpoint_cache = {}
+        self._discovery_cache = {}
         self._lock = threading.Lock()
 
     @abc.abstractmethod
@@ -157,48 +156,104 @@ class BaseIdentityPlugin(plugin.BaseAuthPlugin):
 
         return False
 
-    def get_endpoint(self, session, service_type=None, interface=None,
-                     region_name=None, service_name=None, version=None,
-                     allow={}, **kwargs):
-        """Return a valid endpoint for a service.
+    def get_endpoint_data(self, session, service_type=None, interface=None,
+                          region_name=None, service_name=None, allow={},
+                          allow_version_hack=True, discover_versions=True,
+                          skip_discovery=False, min_version=None,
+                          max_version=None, endpoint_override=None, **kwargs):
+        """Return a valid endpoint data for a service.
 
         If a valid token is not present then a new one will be fetched using
         the session and kwargs.
 
+        version, min_version and max_version can all be given either as a
+        string or a tuple.
+
+        Valid interface types: `public` or `publicURL`,
+                               `internal` or `internalURL`,
+                               `admin` or 'adminURL`
+
         :param session: A session object that can be used for communication.
         :type session: keystoneauth1.session.Session
         :param string service_type: The type of service to lookup the endpoint
                                     for. This plugin will return None (failure)
                                     if service_type is not provided.
-        :param string interface: The exposure of the endpoint. Should be
-                                 `public`, `internal`, `admin`, or `auth`.
-                                 `auth` is special here to use the `auth_url`
-                                 rather than a URL extracted from the service
-                                 catalog. Defaults to `public`.
+        :param interface: Type of endpoint. Can be a single value or a list
+                          of values. If it's a list of values, they will be
+                          looked for in order of preference. Can also be
+                          `keystoneauth1.plugin.AUTH_INTERFACE` to indicate
+                          that the auth_url should be used instead of the
+                          value in the catalog. (optional, defaults to public)
         :param string region_name: The region the endpoint should exist in.
                                    (optional)
         :param string service_name: The name of the service in the catalog.
                                    (optional)
-        :param tuple version: The minimum version number required for this
-                              endpoint. (optional)
         :param dict allow: Extra filters to pass when discovering API
                            versions. (optional)
+        :param bool allow_version_hack: Allow keystoneauth to hack up catalog
+                                        URLS to support older schemes.
+                                        (optional, default True)
+        :param bool discover_versions: Whether to get version metadata from
+                                       the version discovery document even
+                                       if it's not neccessary to fulfill the
+                                       major version request. (optional,
+                                       defaults to True)
+        :param bool skip_discovery: Whether to skip version discovery even
+                                    if a version has been given. This is useful
+                                    if endpoint_override or similar has been
+                                    given and grabbing additional information
+                                    about the endpoint is not useful.
+        :param min_version: The minimum version that is acceptable. Mutually
+                            exclusive with version. If min_version is given
+                            with no max_version it is as if max version is
+                            'latest'. (optional)
+        :param max_version: The maximum version that is acceptable. Mutually
+                            exclusive with version. If min_version is given
+                            with no max_version it is as if max version is
+                            'latest'. (optional)
+        :param str endpoint_override: URL to use instead of looking in the
+                                      catalog. Catalog lookup will be skipped,
+                                      but version discovery will be run.
+                                      Sets allow_version_hack to False
+                                      (optional)
+        :param kwargs: Ignored.
 
         :raises keystoneauth1.exceptions.http.HttpError: An error from an
                                                          invalid HTTP response.
 
-        :return: A valid endpoint URL or None if not available.
-        :rtype: string or None
+        :return: Valid EndpointData or None if not available.
+        :rtype: `keystoneauth1.discover.EndpointData` or None
         """
+        min_version, max_version = discover._normalize_version_args(
+            None, min_version, max_version)
+
         # NOTE(jamielennox): if you specifically ask for requests to be sent to
         # the auth url then we can ignore many of the checks. Typically if you
         # are asking for the auth endpoint it means that there is no catalog to
         # query however we still need to support asking for a specific version
         # of the auth_url for generic plugins.
         if interface is plugin.AUTH_INTERFACE:
-            url = self.auth_url
-            service_type = service_type or 'identity'
-
+            endpoint_data = discover.EndpointData(
+                service_url=self.auth_url,
+                service_type=service_type or 'identity')
+            project_id = None
+        elif endpoint_override:
+            # TODO(mordred) Make a code path that will look for a
+            #               matching entry in the catalog if the catalog
+            #               exists and fill in the interface, region_name, etc.
+            #               For now, just use any information the use has
+            #               provided.
+            endpoint_data = discover.EndpointData(
+                catalog_url=endpoint_override,
+                interface=interface,
+                region_name=region_name,
+                service_name=service_name)
+            # Setting an endpoint_override then calling get_endpoint_data means
+            # you absolutely want the discovery info for the URL in question.
+            # There are no code flows where this will happen for any other
+            # reasons.
+            allow_version_hack = False
+            project_id = self.get_project_id(session)
         else:
             if not service_type:
                 LOG.warning('Plugin cannot return an endpoint without '
@@ -206,52 +261,122 @@ class BaseIdentityPlugin(plugin.BaseAuthPlugin):
                             'service_type to endpoint filtering data.')
                 return None
 
+            # It's possible for things higher in the stack, because of
+            # defaults, to explicitly pass None.
             if not interface:
                 interface = 'public'
 
             service_catalog = self.get_access(session).service_catalog
-            url = service_catalog.url_for(service_type=service_type,
-                                          interface=interface,
-                                          region_name=region_name,
-                                          service_name=service_name)
+            project_id = self.get_project_id(session)
+            # NOTE(mordred): service_catalog.url_data_for raises if it can't
+            # find a match, so this will always be a valid object.
+            endpoint_data = service_catalog.endpoint_data_for(
+                service_type=service_type,
+                interface=interface,
+                region_name=region_name,
+                service_name=service_name)
+            if not endpoint_data:
+                return None
 
-        if not version:
-            # NOTE(jamielennox): This may not be the best thing to default to
-            # but is here for backwards compatibility. It may be worth
-            # defaulting to the most recent version.
-            return url
-
-        # NOTE(jamielennox): For backwards compatibility people might have a
-        # versioned endpoint in their catalog even though they want to use
-        # other endpoint versions. So we support a list of client defined
-        # situations where we can strip the version component from a URL before
-        # doing discovery.
-        hacked_url = discover._get_catalog_discover_hack(service_type, url)
+        if skip_discovery:
+            return endpoint_data
 
         try:
-            disc = self.get_discovery(session, hacked_url, authenticated=False)
+            return endpoint_data.get_versioned_data(
+                session,
+                project_id=project_id,
+                min_version=min_version,
+                max_version=max_version,
+                cache=self._discovery_cache,
+                discover_versions=discover_versions,
+                allow_version_hack=allow_version_hack, allow=allow)
         except (exceptions.DiscoveryFailure,
                 exceptions.HttpError,
                 exceptions.ConnectionError):
-            # NOTE(jamielennox): Again if we can't contact the server we fall
-            # back to just returning the URL from the catalog. This may not be
-            # the best default but we need it for now.
-            LOG.warning('Failed to contact the endpoint at %s for discovery. '
-                        'Fallback to using that endpoint as the base url.',
-                        url)
-        else:
-            # NOTE(jamielennox): urljoin allows the url to be relative or even
-            # protocol-less. The additional trailing '/' make urljoin respect
-            # the current path as canonical even if the url doesn't include it.
-            # for example a "v2" path from http://host/admin should resolve as
-            # http://host/admin/v2 where it would otherwise be host/v2.
-            # This has no effect on absolute urls returned from url_for.
-            url = disc.url_for(version, **allow)
+            # If a version was requested, we didn't find it, return
+            # None.
+            if max_version or min_version:
+                return None
+            # If one wasn't, then the endpoint_data we already have
+            # should be fine
+            return endpoint_data
 
-            if url:
-                url = urllib.parse.urljoin(hacked_url.rstrip('/') + '/', url)
+    def get_endpoint(self, session, service_type=None, interface=None,
+                     region_name=None, service_name=None, version=None,
+                     allow={}, allow_version_hack=True,
+                     skip_discovery=False,
+                     min_version=None, max_version=None,
+                     **kwargs):
+        """Return a valid endpoint for a service.
 
-        return url
+        If a valid token is not present then a new one will be fetched using
+        the session and kwargs.
+
+        version, min_version and max_version can all be given either as a
+        string or a tuple.
+
+        Valid interface types: `public` or `publicURL`,
+                               `internal` or `internalURL`,
+                               `admin` or 'adminURL`
+
+        :param session: A session object that can be used for communication.
+        :type session: keystoneauth1.session.Session
+        :param string service_type: The type of service to lookup the endpoint
+                                    for. This plugin will return None (failure)
+                                    if service_type is not provided.
+        :param interface: Type of endpoint. Can be a single value or a list
+                          of values. If it's a list of values, they will be
+                          looked for in order of preference. Can also be
+                          `keystoneauth1.plugin.AUTH_INTERFACE` to indicate
+                          that the auth_url should be used instead of the
+                          value in the catalog. (optional, defaults to public)
+        :param string region_name: The region the endpoint should exist in.
+                                   (optional)
+        :param string service_name: The name of the service in the catalog.
+                                   (optional)
+        :param version: The minimum version number required for this
+                        endpoint. (optional)
+        :param dict allow: Extra filters to pass when discovering API
+                           versions. (optional)
+        :param bool allow_version_hack: Allow keystoneauth to hack up catalog
+                                        URLS to support older schemes.
+                                        (optional, default True)
+        :param bool skip_discovery: Whether to skip version discovery even
+                                    if a version has been given. This is useful
+                                    if endpoint_override or similar has been
+                                    given and grabbing additional information
+                                    about the endpoint is not useful.
+        :param min_version: The minimum version that is acceptable. Mutually
+                            exclusive with version. If min_version is given
+                            with no max_version it is as if max version is
+                            'latest'. (optional)
+        :param max_version: The maximum version that is acceptable. Mutually
+                            exclusive with version. If min_version is given
+                            with no max_version it is as if max version is
+                            'latest'. (optional)
+
+        :raises keystoneauth1.exceptions.http.HttpError: An error from an
+                                                         invalid HTTP response.
+
+        :return: A valid endpoint URL or None if not available.
+        :rtype: string or None
+        """
+        # Explode `version` into min_version and max_version - everything below
+        # here uses the latter rather than the former.
+        min_version, max_version = discover._normalize_version_args(
+            version, min_version, max_version)
+        # Set discover_versions to False since we're only going to return
+        # a URL. Fetching the microversion data would be needlessly
+        # expensive in the common case. However, discover_versions=False
+        # will still run discovery if the version requested is not the
+        # version in the catalog.
+        endpoint_data = self.get_endpoint_data(
+            session, service_type=service_type, interface=interface,
+            region_name=region_name, service_name=service_name,
+            allow=allow, min_version=min_version, max_version=max_version,
+            discover_versions=False, skip_discovery=skip_discovery,
+            allow_version_hack=allow_version_hack, **kwargs)
+        return endpoint_data.url if endpoint_data else None
 
     def get_user_id(self, session, **kwargs):
         return self.get_access(session).user_id
@@ -298,29 +423,9 @@ class BaseIdentityPlugin(plugin.BaseAuthPlugin):
 
         :returns: A discovery object with the results of looking up that URL.
         """
-        # NOTE(jamielennox): we want to cache endpoints on the session as well
-        # so that they maintain sharing between auth plugins. Create a cache on
-        # the session if it doesn't exist already.
-        try:
-            session_endpoint_cache = session._identity_endpoint_cache
-        except AttributeError:
-            session_endpoint_cache = session._identity_endpoint_cache = {}
-
-        # NOTE(jamielennox): There is a cache located on both the session
-        # object and the auth plugin object so that they can be shared and the
-        # cache is still usable
-        for cache in (self._endpoint_cache, session_endpoint_cache):
-            disc = cache.get(url)
-
-            if disc:
-                break
-        else:
-            disc = discover.Discover(session, url,
-                                     authenticated=authenticated)
-            self._endpoint_cache[url] = disc
-            session_endpoint_cache[url] = disc
-
-        return disc
+        return discover.get_discovery(session=session, url=url,
+                                      cache=self._discovery_cache,
+                                      authenticated=authenticated)
 
     def get_cache_id_elements(self):
         """Get the elements for this auth plugin that make it unique.
